@@ -37,19 +37,20 @@
 #define BC_LANG_H
 
 #include <stdbool.h>
-#if BC_C11
-#include <assert.h>
-#endif // BC_C11
 
+// These have to come first to silence a warning on BC_C11 below.
 #include <status.h>
 #include <vector.h>
 #include <num.h>
 
+#if BC_C11
+#include <assert.h>
+#endif // BC_C11
+
 /// The instructions for bytecode.
-typedef enum BcInst {
-
+typedef enum BcInst
+{
 #if BC_ENABLED
-
 	/// Postfix increment and decrement. Prefix are translated into
 	/// BC_INST_ONE with either BC_INST_ASSIGN_PLUS or BC_INST_ASSIGN_MINUS.
 	BC_INST_INC = 0,
@@ -61,6 +62,7 @@ typedef enum BcInst {
 
 	/// Boolean not.
 	BC_INST_BOOL_NOT,
+
 #if BC_ENABLE_EXTRA_MATH
 	/// Truncation operator.
 	BC_INST_TRUNC,
@@ -75,7 +77,6 @@ typedef enum BcInst {
 	BC_INST_MINUS,
 
 #if BC_ENABLE_EXTRA_MATH
-
 	/// Places operator.
 	BC_INST_PLACES,
 
@@ -328,15 +329,15 @@ typedef enum BcInst {
 } BcInst;
 
 #if BC_C11
-static_assert(BC_INST_INVALID <= UCHAR_MAX,
-              "Too many instructions to fit into an unsigned char");
+_Static_assert(BC_INST_INVALID <= UCHAR_MAX,
+               "Too many instructions to fit into an unsigned char");
 #endif // BC_C11
 
 /// Used by maps to identify where items are in the array.
-typedef struct BcId {
-
+typedef struct BcId
+{
 	/// The name of the item.
-	char *name;
+	char* name;
 
 	/// The index into the array where the item is.
 	size_t idx;
@@ -344,8 +345,8 @@ typedef struct BcId {
 } BcId;
 
 /// The location of a var, array, or array element.
-typedef struct BcLoc {
-
+typedef struct BcLoc
+{
 	/// The index of the var or array.
 	size_t loc;
 
@@ -355,10 +356,10 @@ typedef struct BcLoc {
 } BcLoc;
 
 /// An entry for a constant.
-typedef struct BcConst {
-
+typedef struct BcConst
+{
 	/// The original string as parsed from the source code.
-	char *val;
+	char* val;
 
 	/// The last base that the constant was parsed in.
 	BcBigDig base;
@@ -371,8 +372,8 @@ typedef struct BcConst {
 /// A function. This is also used in dc, not just bc. The reason is that strings
 /// are executed in dc, and they are converted to functions in order to be
 /// executed.
-typedef struct BcFunc {
-
+typedef struct BcFunc
+{
 	/// The bytecode instructions.
 	BcVec code;
 
@@ -398,7 +399,7 @@ typedef struct BcFunc {
 	BcVec consts;
 
 	/// The function's name.
-	const char *name;
+	const char* name;
 
 #if BC_ENABLED
 	/// True if the function is a void function.
@@ -408,8 +409,8 @@ typedef struct BcFunc {
 } BcFunc;
 
 /// Types of results that can be pushed onto the results stack.
-typedef enum BcResultType {
-
+typedef enum BcResultType
+{
 	/// Result is a variable.
 	BC_RESULT_VAR,
 
@@ -463,8 +464,8 @@ typedef enum BcResultType {
 } BcResultType;
 
 /// A union to store data for various result types.
-typedef union BcResultData {
-
+typedef union BcResultData
+{
 	/// A number. Strings are stored here too; they are numbers with
 	/// cap == 0 && num == NULL. The string's index into the strings vector is
 	/// stored in the scale field. But this is only used for strings stored in
@@ -481,8 +482,8 @@ typedef union BcResultData {
 } BcResultData;
 
 /// A tagged union for results.
-typedef struct BcResult {
-
+typedef struct BcResult
+{
 	/// The tag. The type of the result.
 	BcResultType t;
 
@@ -493,8 +494,8 @@ typedef struct BcResult {
 
 /// An instruction pointer. This is how bc knows where in the bytecode vector,
 /// and which function, the current execution is.
-typedef struct BcInstPtr {
-
+typedef struct BcInstPtr
+{
 	/// The index of the currently executing function in the fns vector.
 	size_t func;
 
@@ -509,8 +510,8 @@ typedef struct BcInstPtr {
 } BcInstPtr;
 
 /// Types of identifiers.
-typedef enum BcType {
-
+typedef enum BcType
+{
 	/// Variable.
 	BC_TYPE_VAR,
 
@@ -528,8 +529,8 @@ typedef enum BcType {
 
 #if BC_ENABLED
 /// An auto variable in bc.
-typedef struct BcAuto {
-
+typedef struct BcAuto
+{
 	/// The index of the variable in the vars or arrs vectors.
 	size_t idx;
 
@@ -548,7 +549,8 @@ struct BcProgram;
  * @param name  The name of the function. The string is assumed to be owned by
  *              some other entity.
  */
-void bc_func_init(BcFunc *f, const char* name);
+void
+bc_func_init(BcFunc* f, const char* name);
 
 /**
  * Inserts an auto into the function.
@@ -559,15 +561,17 @@ void bc_func_init(BcFunc *f, const char* name);
  * @param line  The line in the source code where the insert happened. This is
  *              solely for error reporting.
  */
-void bc_func_insert(BcFunc *f, struct BcProgram* p, char* name,
-                    BcType type, size_t line);
+void
+bc_func_insert(BcFunc* f, struct BcProgram* p, char* name, BcType type,
+               size_t line);
 
 /**
  * Resets a function in preparation for it to be reused. This can happen in bc
  * because it is a dynamic language and functions can be redefined.
  * @param f  The functio to reset.
  */
-void bc_func_reset(BcFunc *f);
+void
+bc_func_reset(BcFunc* f);
 
 #ifndef NDEBUG
 /**
@@ -576,7 +580,8 @@ void bc_func_reset(BcFunc *f);
  * check for memory leaks.
  * @param func  The function to free as a void pointer.
  */
-void bc_func_free(void *func);
+void
+bc_func_free(void* func);
 #endif // NDEBUG
 
 /**
@@ -590,7 +595,8 @@ void bc_func_free(void *func);
  * @param nums  True if the array should be for numbers, false if it should be
  *              for vectors.
  */
-void bc_array_init(BcVec *a, bool nums);
+void
+bc_array_init(BcVec* a, bool nums);
 
 /**
  * Copies an array to another array. This is used to do pass arrays to functions
@@ -599,19 +605,22 @@ void bc_array_init(BcVec *a, bool nums);
  * @param d  The destination array.
  * @param s  The source array.
  */
-void bc_array_copy(BcVec *d, const BcVec *s);
+void
+bc_array_copy(BcVec* d, const BcVec* s);
 
 /**
  * Frees a string stored in a function. This is a destructor.
  * @param string  The string to free as a void pointer.
  */
-void bc_string_free(void *string);
+void
+bc_string_free(void* string);
 
 /**
  * Frees a constant stored in a function. This is a destructor.
  * @param constant  The constant to free as a void pointer.
  */
-void bc_const_free(void *constant);
+void
+bc_const_free(void* constant);
 
 /**
  * Clears a result. It sets the type to BC_RESULT_TEMP and clears the union by
@@ -619,7 +628,8 @@ void bc_const_free(void *constant);
  * uninitialized data.
  * @param r  The result to clear.
  */
-void bc_result_clear(BcResult *r);
+void
+bc_result_clear(BcResult* r);
 
 /**
  * Copies a result into another. This is done for things like duplicating the
@@ -628,13 +638,15 @@ void bc_result_clear(BcResult *r);
  * @param d    The destination result.
  * @param src  The source result.
  */
-void bc_result_copy(BcResult *d, BcResult *src);
+void
+bc_result_copy(BcResult* d, BcResult* src);
 
 /**
  * Frees a result. This is a destructor.
  * @param result  The result to free as a void pointer.
  */
-void bc_result_free(void *result);
+void
+bc_result_free(void* result);
 
 /**
  * Expands an array to @a len. This can happen because in bc, you do not have to
@@ -645,7 +657,8 @@ void bc_result_free(void *result);
  * @param a    The array to expand.
  * @param len  The length to expand to.
  */
-void bc_array_expand(BcVec *a, size_t len);
+void
+bc_array_expand(BcVec* a, size_t len);
 
 /**
  * Compare two BcId's and return the result. Since they are just comparing the
@@ -655,7 +668,8 @@ void bc_array_expand(BcVec *a, size_t len);
  * @param e2  The second id.
  * @return    The result of strcmp() on the BcId's names.
  */
-int bc_id_cmp(const BcId *e1, const BcId *e2);
+int
+bc_id_cmp(const BcId* e1, const BcId* e2);
 
 #if BC_ENABLED
 
