@@ -45,165 +45,319 @@
 #include <num.h>
 #include <vm.h>
 
+#ifndef _WIN32
+#include <pthread.h>
+#endif // _WIN32
+
 // The asserts in this file are important to testing; in many cases, the test
 // would not work without the asserts, so don't remove them without reason.
 //
 // Also, there are many uses of bc_num_clear() here; that is because numbers are
 // being reused, and a clean slate is required.
 //
-// Also, there are a bunch of BC_UNSETJMP and BC_SETJMP_LOCKED() between calls
-// to bc_num_init(). That is because locals are being initialized, and unlike bc
-// proper, this code cannot assume that allocation failures are fatal. So we
-// have to reset the jumps every time to ensure that the locals will be correct
-// after jumping.
+// Also, there are a bunch of BC_UNSETJMP between calls to bc_num_init(). That
+// is because locals are being initialized, and unlike bc proper, this code
+// cannot assume that allocation failures are fatal. So we have to reset the
+// jumps every time to ensure that the locals will be correct after jumping.
 
-void bcl_handleSignal(void) {
+static BclTls* tls = NULL;
+static BclTls tls_real;
 
-	// Signal already in flight, or bc is not executing.
-	if (vm.sig || !vm.running) return;
+BclError
+bcl_start(void)
+{
+#ifndef _WIN32
 
-	vm.sig = 1;
+	int r;
 
-	assert(vm.jmp_bufs.len);
+	if (tls != NULL) return BCL_ERROR_NONE;
 
-	if (!vm.sig_lock) BC_JMP;
+	r = pthread_key_create(&tls_real, NULL);
+	if (BC_ERR(r != 0)) return BCL_ERROR_FATAL_ALLOC_ERR;
+
+#else // _WIN32
+
+	if (tls != NULL) return BCL_ERROR_NONE;
+
+	tls_real = TlsAlloc();
+	if (BC_ERR(tls_real == TLS_OUT_OF_INDEXES))
+	{
+		return BCL_ERROR_FATAL_ALLOC_ERR;
+	}
+
+#endif // _WIN32
+
+	tls = &tls_real;
+
+	return BCL_ERROR_NONE;
 }
 
-bool bcl_running(void) {
-	return vm.running != 0;
+/**
+ * Sets the thread-specific data for the thread.
+ * @param vm  The @a BcVm to set as the thread data.
+ * @return    An error code, if any.
+ */
+static BclError
+bcl_setspecific(BcVm* vm)
+{
+#ifndef _WIN32
+
+	int r;
+
+	assert(tls != NULL);
+
+	r = pthread_setspecific(tls_real, vm);
+	if (BC_ERR(r != 0)) return BCL_ERROR_FATAL_ALLOC_ERR;
+
+#else // _WIN32
+
+	bool r;
+
+	assert(tls != NULL);
+
+	r = TlsSetValue(tls_real, vm);
+	if (BC_ERR(!r)) return BCL_ERROR_FATAL_ALLOC_ERR;
+
+#endif // _WIN32
+
+	return BCL_ERROR_NONE;
 }
 
-BclError bcl_init(void) {
+BcVm*
+bcl_getspecific(void)
+{
+	BcVm* vm;
 
+#ifndef _WIN32
+
+	vm = pthread_getspecific(tls_real);
+
+#else // _WIN32
+
+	vm = TlsGetValue(tls_real);
+
+#endif // _WIN32
+
+	return vm;
+}
+
+BclError
+bcl_init(void)
+{
 	BclError e = BCL_ERROR_NONE;
+	BcVm* vm;
 
-	BC_SIG_LOCK;
+	assert(tls != NULL);
 
-	vm.refs += 1;
+	vm = bcl_getspecific();
+	if (vm != NULL)
+	{
+		assert(vm->refs >= 1);
+
+		vm->refs += 1;
 
-	if (vm.refs > 1) {
-		BC_SIG_UNLOCK;
 		return e;
 	}
 
+	vm = bc_vm_malloc(sizeof(BcVm));
+	if (BC_ERR(vm == NULL)) return BCL_ERROR_FATAL_ALLOC_ERR;
+
+	e = bcl_setspecific(vm);
+	if (BC_ERR(e != BCL_ERROR_NONE))
+	{
+		free(vm);
+		return e;
+	}
+
+	memset(vm, 0, sizeof(BcVm));
+
+	vm->refs += 1;
+
+	assert(vm->refs == 1);
+
 	// Setting these to NULL ensures that if an error occurs, we only free what
 	// is necessary.
-	vm.ctxts.v = NULL;
-	vm.jmp_bufs.v = NULL;
-	vm.out.v = NULL;
+	vm->ctxts.v = NULL;
+	vm->jmp_bufs.v = NULL;
+	vm->out.v = NULL;
 
-	vm.abrt = false;
+	vm->abrt = false;
+	vm->leading_zeroes = false;
+	vm->digit_clamp = true;
 
 	// The jmp_bufs always has to be initialized first.
-	bc_vec_init(&vm.jmp_bufs, sizeof(sigjmp_buf), BC_DTOR_NONE);
+	bc_vec_init(&vm->jmp_bufs, sizeof(sigjmp_buf), BC_DTOR_NONE);
 
-	BC_FUNC_HEADER_INIT(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vm_init();
 
-	bc_vec_init(&vm.ctxts, sizeof(BclContext), BC_DTOR_NONE);
-	bc_vec_init(&vm.out, sizeof(uchar), BC_DTOR_NONE);
+	bc_vec_init(&vm->ctxts, sizeof(BclContext), BC_DTOR_NONE);
+	bc_vec_init(&vm->out, sizeof(uchar), BC_DTOR_NONE);
 
 	// We need to seed this in case /dev/random and /dev/urandm don't work.
 	srand((unsigned int) time(NULL));
-	bc_rand_init(&vm.rng);
+	bc_rand_init(&vm->rng);
 
 err:
+
 	// This is why we had to set them to NULL.
-	if (BC_ERR(vm.err)) {
-		if (vm.out.v != NULL) bc_vec_free(&vm.out);
-		if (vm.jmp_bufs.v != NULL) bc_vec_free(&vm.jmp_bufs);
-		if (vm.ctxts.v != NULL) bc_vec_free(&vm.ctxts);
+	if (BC_ERR(vm != NULL && vm->err))
+	{
+		if (vm->out.v != NULL) bc_vec_free(&vm->out);
+		if (vm->jmp_bufs.v != NULL) bc_vec_free(&vm->jmp_bufs);
+		if (vm->ctxts.v != NULL) bc_vec_free(&vm->ctxts);
+		bcl_setspecific(NULL);
+		free(vm);
 	}
 
-	BC_FUNC_FOOTER_UNLOCK(e);
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER(vm, e);
 
 	return e;
 }
 
-BclError bcl_pushContext(BclContext ctxt) {
-
+BclError
+bcl_pushContext(BclContext ctxt)
+{
 	BclError e = BCL_ERROR_NONE;
+	BcVm* vm = bcl_getspecific();
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
-	bc_vec_push(&vm.ctxts, &ctxt);
+	bc_vec_push(&vm->ctxts, &ctxt);
 
 err:
-	BC_FUNC_FOOTER_UNLOCK(e);
+	BC_FUNC_FOOTER(vm, e);
 	return e;
 }
 
-void bcl_popContext(void) {
-	if (vm.ctxts.len) bc_vec_pop(&vm.ctxts);
+void
+bcl_popContext(void)
+{
+	BcVm* vm = bcl_getspecific();
+
+	if (vm->ctxts.len) bc_vec_pop(&vm->ctxts);
 }
 
-BclContext bcl_context(void) {
-	if (!vm.ctxts.len) return NULL;
-	return *((BclContext*) bc_vec_top(&vm.ctxts));
+static BclContext
+bcl_contextHelper(BcVm* vm)
+{
+	if (!vm->ctxts.len) return NULL;
+	return *((BclContext*) bc_vec_top(&vm->ctxts));
 }
 
-void bcl_free(void) {
+BclContext
+bcl_context(void)
+{
+	BcVm* vm = bcl_getspecific();
+	return bcl_contextHelper(vm);
+}
 
+void
+bcl_free(void)
+{
 	size_t i;
+	BcVm* vm = bcl_getspecific();
 
-	BC_SIG_LOCK;
+	vm->refs -= 1;
+	if (vm->refs) return;
 
-	vm.refs -= 1;
+	bc_rand_free(&vm->rng);
+	bc_vec_free(&vm->out);
 
-	if (vm.refs) {
-		BC_SIG_UNLOCK;
-		return;
-	}
-
-	bc_rand_free(&vm.rng);
-	bc_vec_free(&vm.out);
-
-	for (i = 0; i < vm.ctxts.len; ++i) {
-		BclContext ctxt = *((BclContext*) bc_vec_item(&vm.ctxts, i));
+	for (i = 0; i < vm->ctxts.len; ++i)
+	{
+		BclContext ctxt = *((BclContext*) bc_vec_item(&vm->ctxts, i));
 		bcl_ctxt_free(ctxt);
 	}
 
-	bc_vec_free(&vm.ctxts);
+	bc_vec_free(&vm->ctxts);
 
 	bc_vm_atexit();
 
-	BC_SIG_UNLOCK;
-
-	memset(&vm, 0, sizeof(BcVm));
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	free(vm);
+	bcl_setspecific(NULL);
 }
 
-void bcl_gc(void) {
-	BC_SIG_LOCK;
+void
+bcl_end(void)
+{
+#ifndef _WIN32
+
+	// We ignore the return value.
+	pthread_key_delete(tls_real);
+
+#else // _WIN32
+
+	// We ignore the return value.
+	TlsFree(tls_real);
+
+#endif // _WIN32
+
+	tls = NULL;
+}
+
+void
+bcl_gc(void)
+{
 	bc_vm_freeTemps();
-	BC_SIG_UNLOCK;
 }
 
-bool bcl_abortOnFatalError(void) {
-	return vm.abrt;
+bool
+bcl_abortOnFatalError(void)
+{
+	BcVm* vm = bcl_getspecific();
+
+	return vm->abrt;
 }
 
-void bcl_setAbortOnFatalError(bool abrt) {
-	vm.abrt = abrt;
+void
+bcl_setAbortOnFatalError(bool abrt)
+{
+	BcVm* vm = bcl_getspecific();
+
+	vm->abrt = abrt;
 }
 
-bool bcl_leadingZeroes(void) {
-	return vm.leading_zeroes;
+bool
+bcl_leadingZeroes(void)
+{
+	BcVm* vm = bcl_getspecific();
+
+	return vm->leading_zeroes;
 }
 
-void bcl_setLeadingZeroes(bool leadingZeroes) {
-	vm.leading_zeroes = leadingZeroes;
+void
+bcl_setLeadingZeroes(bool leadingZeroes)
+{
+	BcVm* vm = bcl_getspecific();
+
+	vm->leading_zeroes = leadingZeroes;
 }
 
-BclContext bcl_ctxt_create(void) {
+bool
+bcl_digitClamp(void)
+{
+	BcVm* vm = bcl_getspecific();
 
+	return vm->digit_clamp;
+}
+
+void
+bcl_setDigitClamp(bool digitClamp)
+{
+	BcVm* vm = bcl_getspecific();
+
+	vm->digit_clamp = digitClamp;
+}
+
+BclContext
+bcl_ctxt_create(void)
+{
+	BcVm* vm = bcl_getspecific();
 	BclContext ctxt = NULL;
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	// We want the context to be free of any interference of other parties, so
 	// malloc() is appropriate here.
@@ -214,70 +368,87 @@ BclContext bcl_ctxt_create(void) {
 
 	ctxt->scale = 0;
 	ctxt->ibase = 10;
-	ctxt->obase= 10;
+	ctxt->obase = 10;
 
 err:
-	if (BC_ERR(vm.err && ctxt != NULL)) {
+
+	if (BC_ERR(vm->err && ctxt != NULL))
+	{
 		if (ctxt->nums.v != NULL) bc_vec_free(&ctxt->nums);
 		free(ctxt);
 		ctxt = NULL;
 	}
 
-	BC_FUNC_FOOTER_NO_ERR;
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER_NO_ERR(vm);
 
 	return ctxt;
 }
 
-void bcl_ctxt_free(BclContext ctxt) {
-	BC_SIG_LOCK;
+void
+bcl_ctxt_free(BclContext ctxt)
+{
 	bc_vec_free(&ctxt->free_nums);
 	bc_vec_free(&ctxt->nums);
 	free(ctxt);
-	BC_SIG_UNLOCK;
 }
 
-void bcl_ctxt_freeNums(BclContext ctxt) {
+void
+bcl_ctxt_freeNums(BclContext ctxt)
+{
 	bc_vec_popAll(&ctxt->nums);
 	bc_vec_popAll(&ctxt->free_nums);
 }
 
-size_t bcl_ctxt_scale(BclContext ctxt) {
+size_t
+bcl_ctxt_scale(BclContext ctxt)
+{
 	return ctxt->scale;
 }
 
-void bcl_ctxt_setScale(BclContext ctxt, size_t scale) {
+void
+bcl_ctxt_setScale(BclContext ctxt, size_t scale)
+{
 	ctxt->scale = scale;
 }
 
-size_t bcl_ctxt_ibase(BclContext ctxt) {
+size_t
+bcl_ctxt_ibase(BclContext ctxt)
+{
 	return ctxt->ibase;
 }
 
-void bcl_ctxt_setIbase(BclContext ctxt, size_t ibase) {
+void
+bcl_ctxt_setIbase(BclContext ctxt, size_t ibase)
+{
 	if (ibase < BC_NUM_MIN_BASE) ibase = BC_NUM_MIN_BASE;
 	else if (ibase > BC_NUM_MAX_IBASE) ibase = BC_NUM_MAX_IBASE;
 	ctxt->ibase = ibase;
 }
 
-size_t bcl_ctxt_obase(BclContext ctxt) {
+size_t
+bcl_ctxt_obase(BclContext ctxt)
+{
 	return ctxt->obase;
 }
 
-void bcl_ctxt_setObase(BclContext ctxt, size_t obase) {
+void
+bcl_ctxt_setObase(BclContext ctxt, size_t obase)
+{
 	ctxt->obase = obase;
 }
 
-BclError bcl_err(BclNumber n) {
-
+BclError
+bcl_err(BclNumber n)
+{
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 
 	// Errors are encoded as (0 - error_code). If the index is in that range, it
 	// is an encoded error.
-	if (n.i >= ctxt->nums.len) {
+	if (n.i >= ctxt->nums.len)
+	{
 		if (n.i > 0 - (size_t) BCL_ERROR_NELEMS) return (BclError) (0 - n.i);
 		else return BCL_ERROR_INVALID_NUM;
 	}
@@ -290,14 +461,15 @@ BclError bcl_err(BclNumber n) {
  * @param n     The BcNum to insert.
  * @return      The resulting BclNumber from the insert.
  */
-static BclNumber bcl_num_insert(BclContext ctxt, BcNum *restrict n) {
-
+static BclNumber
+bcl_num_insert(BclContext ctxt, BcNum* restrict n)
+{
 	BclNumber idx;
 
 	// If there is a free spot...
-	if (ctxt->free_nums.len) {
-
-		BcNum *ptr;
+	if (ctxt->free_nums.len)
+	{
+		BcNum* ptr;
 
 		// Get the index of the free spot and remove it.
 		idx = *((BclNumber*) bc_vec_top(&ctxt->free_nums));
@@ -307,38 +479,37 @@ static BclNumber bcl_num_insert(BclContext ctxt, BcNum *restrict n) {
 		ptr = bc_vec_item(&ctxt->nums, idx.i);
 		memcpy(ptr, n, sizeof(BcNum));
 	}
-	else {
+	else
+	{
 		// Just push the number onto the vector.
 		idx.i = ctxt->nums.len;
 		bc_vec_push(&ctxt->nums, n);
 	}
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-BclNumber bcl_num_create(void) {
-
+BclNumber
+bcl_num_create(void)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum n;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
 	bc_num_init(&n, BC_NUM_DEF_SIZE);
 
 err:
-	BC_FUNC_FOOTER_UNLOCK(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, n, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
@@ -348,43 +519,43 @@ err:
  * @param n     The index of the number.
  * @param num   The number to destroy.
  */
-static void bcl_num_dtor(BclContext ctxt, BclNumber n, BcNum *restrict num) {
-
-	BC_SIG_ASSERT_LOCKED;
-
+static void
+bcl_num_dtor(BclContext ctxt, BclNumber n, BcNum* restrict num)
+{
 	assert(num != NULL && num->num != NULL);
 
 	bcl_num_destruct(num);
 	bc_vec_push(&ctxt->free_nums, &n);
 }
 
-void bcl_num_free(BclNumber n) {
-
-	BcNum *num;
+void
+bcl_num_free(BclNumber n)
+{
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
-
-	BC_SIG_LOCK;
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
 	num = BC_NUM(ctxt, n);
 
 	bcl_num_dtor(ctxt, n, num);
-
-	BC_SIG_UNLOCK;
 }
 
-BclError bcl_copy(BclNumber d, BclNumber s) {
-
+BclError
+bcl_copy(BclNumber d, BclNumber s)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *dest, *src;
+	BcNum* dest;
+	BcNum* src;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	assert(d.i < ctxt->nums.len && s.i < ctxt->nums.len);
 
@@ -397,23 +568,23 @@ BclError bcl_copy(BclNumber d, BclNumber s) {
 	bc_num_copy(dest, src);
 
 err:
-	BC_FUNC_FOOTER_UNLOCK(e);
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER(vm, e);
 
 	return e;
 }
 
-BclNumber bcl_dup(BclNumber s) {
-
+BclNumber
+bcl_dup(BclNumber s)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum *src, dest;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -428,17 +599,16 @@ BclNumber bcl_dup(BclNumber s) {
 	bc_num_createCopy(&dest, src);
 
 err:
-	BC_FUNC_FOOTER_UNLOCK(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, dest, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-void bcl_num_destruct(void *num) {
-
-	BcNum *n = (BcNum*) num;
+void
+bcl_num_destruct(void* num)
+{
+	BcNum* n = (BcNum*) num;
 
 	assert(n != NULL);
 
@@ -448,12 +618,14 @@ void bcl_num_destruct(void *num) {
 	bc_num_clear(num);
 }
 
-bool bcl_num_neg(BclNumber n) {
-
-	BcNum *num;
+bool
+bcl_num_neg(BclNumber n)
+{
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -464,12 +636,14 @@ bool bcl_num_neg(BclNumber n) {
 	return BC_NUM_NEG(num) != 0;
 }
 
-void bcl_num_setNeg(BclNumber n, bool neg) {
-
-	BcNum *num;
+void
+bcl_num_setNeg(BclNumber n, bool neg)
+{
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -480,12 +654,14 @@ void bcl_num_setNeg(BclNumber n, bool neg) {
 	num->rdx = BC_NUM_NEG_VAL(num, neg);
 }
 
-size_t bcl_num_scale(BclNumber n) {
-
-	BcNum *num;
+size_t
+bcl_num_scale(BclNumber n)
+{
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -496,17 +672,19 @@ size_t bcl_num_scale(BclNumber n) {
 	return bc_num_scale(num);
 }
 
-BclError bcl_num_setScale(BclNumber n, size_t scale) {
-
+BclError
+bcl_num_setScale(BclNumber n, size_t scale)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *nptr;
+	BcNum* nptr;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 
 	BC_CHECK_NUM_ERR(ctxt, n);
 
-	BC_FUNC_HEADER(err);
+	BC_FUNC_HEADER(vm, err);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -518,20 +696,19 @@ BclError bcl_num_setScale(BclNumber n, size_t scale) {
 	else if (scale < nptr->scale) bc_num_truncate(nptr, nptr->scale - scale);
 
 err:
-	BC_SIG_MAYLOCK;
-	BC_FUNC_FOOTER(e);
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER(vm, e);
 
 	return e;
 }
 
-size_t bcl_num_len(BclNumber n) {
-
-	BcNum *num;
+size_t
+bcl_num_len(BclNumber n)
+{
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -542,15 +719,17 @@ size_t bcl_num_len(BclNumber n) {
 	return bc_num_len(num);
 }
 
-BclError bcl_bigdig(BclNumber n, BclBigDig *result) {
-
+BclError
+bcl_bigdig(BclNumber n, BclBigDig* result)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *num;
+	BcNum* num;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	assert(n.i < ctxt->nums.len);
 	assert(result != NULL);
@@ -563,34 +742,32 @@ BclError bcl_bigdig(BclNumber n, BclBigDig *result) {
 
 err:
 	bcl_num_dtor(ctxt, n, num);
-	BC_FUNC_FOOTER_UNLOCK(e);
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER(vm, e);
 
 	return e;
 }
 
-BclNumber bcl_bigdig2num(BclBigDig val) {
-
+BclNumber
+bcl_bigdig2num(BclBigDig val)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum n;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
 	bc_num_createFromBigdig(&n, val);
 
 err:
-	BC_FUNC_FOOTER_UNLOCK(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, n, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
@@ -602,21 +779,24 @@ err:
  * @param req   The function to get the size of the result for preallocation.
  * @return      The result of the operation.
  */
-static BclNumber bcl_binary(BclNumber a, BclNumber b, const BcNumBinaryOp op,
-                            const BcNumBinaryOpReq req)
+static BclNumber
+bcl_binary(BclNumber a, BclNumber b, const BcNumBinaryOp op,
+           const BcNumBinaryOpReq req)
 {
 	BclError e = BCL_ERROR_NONE;
-	BcNum *aptr, *bptr;
+	BcNum* aptr;
+	BcNum* bptr;
 	BcNum c;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
 	BC_CHECK_NUM(ctxt, a);
 	BC_CHECK_NUM(ctxt, b);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -632,71 +812,83 @@ static BclNumber bcl_binary(BclNumber a, BclNumber b, const BcNumBinaryOp op,
 	bc_num_clear(&c);
 	bc_num_init(&c, req(aptr, bptr, ctxt->scale));
 
-	BC_SIG_UNLOCK;
-
 	op(aptr, bptr, &c, ctxt->scale);
 
 err:
 
-	BC_SIG_MAYLOCK;
-
 	// Eat the operands.
 	bcl_num_dtor(ctxt, a, aptr);
 	if (b.i != a.i) bcl_num_dtor(ctxt, b, bptr);
 
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, c, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-BclNumber bcl_add(BclNumber a, BclNumber b) {
+BclNumber
+bcl_add(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_add, bc_num_addReq);
 }
 
-BclNumber bcl_sub(BclNumber a, BclNumber b) {
+BclNumber
+bcl_sub(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_sub, bc_num_addReq);
 }
 
-BclNumber bcl_mul(BclNumber a, BclNumber b) {
+BclNumber
+bcl_mul(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_mul, bc_num_mulReq);
 }
 
-BclNumber bcl_div(BclNumber a, BclNumber b) {
+BclNumber
+bcl_div(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_div, bc_num_divReq);
 }
 
-BclNumber bcl_mod(BclNumber a, BclNumber b) {
+BclNumber
+bcl_mod(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_mod, bc_num_divReq);
 }
 
-BclNumber bcl_pow(BclNumber a, BclNumber b) {
+BclNumber
+bcl_pow(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_pow, bc_num_powReq);
 }
 
-BclNumber bcl_lshift(BclNumber a, BclNumber b) {
+BclNumber
+bcl_lshift(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_lshift, bc_num_placesReq);
 }
 
-BclNumber bcl_rshift(BclNumber a, BclNumber b) {
+BclNumber
+bcl_rshift(BclNumber a, BclNumber b)
+{
 	return bcl_binary(a, b, bc_num_rshift, bc_num_placesReq);
 }
 
-BclNumber bcl_sqrt(BclNumber a) {
-
+BclNumber
+bcl_sqrt(BclNumber a)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *aptr;
+	BcNum* aptr;
 	BcNum b;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
 	BC_CHECK_NUM(ctxt, a);
 
-	BC_FUNC_HEADER(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -707,30 +899,30 @@ BclNumber bcl_sqrt(BclNumber a) {
 	bc_num_sqrt(aptr, &b, ctxt->scale);
 
 err:
-	BC_SIG_MAYLOCK;
 	bcl_num_dtor(ctxt, a, aptr);
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, b, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-BclError bcl_divmod(BclNumber a, BclNumber b, BclNumber *c, BclNumber *d) {
-
+BclError
+bcl_divmod(BclNumber a, BclNumber b, BclNumber* c, BclNumber* d)
+{
 	BclError e = BCL_ERROR_NONE;
 	size_t req;
-	BcNum *aptr, *bptr;
+	BcNum* aptr;
+	BcNum* bptr;
 	BcNum cnum, dnum;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 
 	BC_CHECK_NUM_ERR(ctxt, a);
 	BC_CHECK_NUM_ERR(ctxt, b);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 2);
 
@@ -749,24 +941,21 @@ BclError bcl_divmod(BclNumber a, BclNumber b, BclNumber *c, BclNumber *d) {
 
 	// Initialize the numbers.
 	bc_num_init(&cnum, req);
-	BC_UNSETJMP;
-	BC_SETJMP_LOCKED(err);
+	BC_UNSETJMP(vm);
+	BC_SETJMP(vm, err);
 	bc_num_init(&dnum, req);
 
-	BC_SIG_UNLOCK;
-
 	bc_num_divmod(aptr, bptr, &cnum, &dnum, ctxt->scale);
 
 err:
-	BC_SIG_MAYLOCK;
 
 	// Eat the operands.
 	bcl_num_dtor(ctxt, a, aptr);
 	if (b.i != a.i) bcl_num_dtor(ctxt, b, bptr);
 
 	// If there was an error...
-	if (BC_ERR(vm.err)) {
-
+	if (BC_ERR(vm->err))
+	{
 		// Free the results.
 		if (cnum.num != NULL) bc_num_free(&cnum);
 		if (dnum.num != NULL) bc_num_free(&dnum);
@@ -775,38 +964,40 @@ err:
 		c->i = 0 - (size_t) BCL_ERROR_INVALID_NUM;
 		d->i = c->i;
 
-		BC_FUNC_FOOTER(e);
+		BC_FUNC_FOOTER(vm, e);
 	}
-	else {
-
-		BC_FUNC_FOOTER(e);
+	else
+	{
+		BC_FUNC_FOOTER(vm, e);
 
 		// Insert the results into the context.
 		*c = bcl_num_insert(ctxt, &cnum);
 		*d = bcl_num_insert(ctxt, &dnum);
 	}
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return e;
 }
 
-BclNumber bcl_modexp(BclNumber a, BclNumber b, BclNumber c) {
-
+BclNumber
+bcl_modexp(BclNumber a, BclNumber b, BclNumber c)
+{
 	BclError e = BCL_ERROR_NONE;
 	size_t req;
-	BcNum *aptr, *bptr, *cptr;
+	BcNum* aptr;
+	BcNum* bptr;
+	BcNum* cptr;
 	BcNum d;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
 	BC_CHECK_NUM(ctxt, a);
 	BC_CHECK_NUM(ctxt, b);
 	BC_CHECK_NUM(ctxt, c);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -828,32 +1019,30 @@ BclNumber bcl_modexp(BclNumber a, BclNumber b, BclNumber c) {
 	// Initialize the result.
 	bc_num_init(&d, req);
 
-	BC_SIG_UNLOCK;
-
 	bc_num_modexp(aptr, bptr, cptr, &d);
 
 err:
-	BC_SIG_MAYLOCK;
 
 	// Eat the operands.
 	bcl_num_dtor(ctxt, a, aptr);
 	if (b.i != a.i) bcl_num_dtor(ctxt, b, bptr);
 	if (c.i != a.i && c.i != b.i) bcl_num_dtor(ctxt, c, cptr);
 
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, d, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-ssize_t bcl_cmp(BclNumber a, BclNumber b) {
-
-	BcNum *aptr, *bptr;
+ssize_t
+bcl_cmp(BclNumber a, BclNumber b)
+{
+	BcNum* aptr;
+	BcNum* bptr;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(a.i < ctxt->nums.len && b.i < ctxt->nums.len);
 
@@ -866,12 +1055,14 @@ ssize_t bcl_cmp(BclNumber a, BclNumber b) {
 	return bc_num_cmp(aptr, bptr);
 }
 
-void bcl_zero(BclNumber n) {
-
-	BcNum *nptr;
+void
+bcl_zero(BclNumber n)
+{
+	BcNum* nptr;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -882,12 +1073,14 @@ void bcl_zero(BclNumber n) {
 	bc_num_zero(nptr);
 }
 
-void bcl_one(BclNumber n) {
-
-	BcNum *nptr;
+void
+bcl_one(BclNumber n)
+{
+	BcNum* nptr;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -898,17 +1091,19 @@ void bcl_one(BclNumber n) {
 	bc_num_one(nptr);
 }
 
-BclNumber bcl_parse(const char *restrict val) {
-
+BclNumber
+bcl_parse(const char* restrict val)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum n;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 	bool neg;
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -920,8 +1115,9 @@ BclNumber bcl_parse(const char *restrict val) {
 
 	if (neg) val += 1;
 
-	if (!bc_num_strValid(val)) {
-		vm.err = BCL_ERROR_PARSE_INVALID_STR;
+	if (!bc_num_strValid(val))
+	{
+		vm->err = BCL_ERROR_PARSE_INVALID_STR;
 		goto err;
 	}
 
@@ -929,34 +1125,31 @@ BclNumber bcl_parse(const char *restrict val) {
 	bc_num_clear(&n);
 	bc_num_init(&n, BC_NUM_DEF_SIZE);
 
-	BC_SIG_UNLOCK;
-
 	bc_num_parse(&n, val, (BcBigDig) ctxt->ibase);
 
 	// Set the negative.
 	n.rdx = BC_NUM_NEG_VAL_NP(n, neg);
 
 err:
-	BC_SIG_MAYLOCK;
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, n, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-char* bcl_string(BclNumber n) {
-
-	BcNum *nptr;
-	char *str = NULL;
+char*
+bcl_string(BclNumber n)
+{
+	BcNum* nptr;
+	char* str = NULL;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ASSERT(ctxt);
+	BC_CHECK_CTXT_ASSERT(vm, ctxt);
 
 	if (BC_ERR(n.i >= ctxt->nums.len)) return str;
 
-	BC_FUNC_HEADER(err);
+	BC_FUNC_HEADER(vm, err);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -965,42 +1158,40 @@ char* bcl_string(BclNumber n) {
 	assert(nptr != NULL && nptr->num != NULL);
 
 	// Clear the buffer.
-	bc_vec_popAll(&vm.out);
+	bc_vec_popAll(&vm->out);
 
 	// Print to the buffer.
 	bc_num_print(nptr, (BcBigDig) ctxt->obase, false);
-	bc_vec_pushByte(&vm.out, '\0');
-
-	BC_SIG_LOCK;
+	bc_vec_pushByte(&vm->out, '\0');
 
 	// Just dup the string; the caller is responsible for it.
-	str = bc_vm_strdup(vm.out.v);
+	str = bc_vm_strdup(vm->out.v);
 
 err:
 
 	// Eat the operand.
 	bcl_num_dtor(ctxt, n, nptr);
 
-	BC_FUNC_FOOTER_NO_ERR;
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
+	BC_FUNC_FOOTER_NO_ERR(vm);
 
 	return str;
 }
 
-BclNumber bcl_irand(BclNumber a) {
-
+BclNumber
+bcl_irand(BclNumber a)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *aptr;
+	BcNum* aptr;
 	BcNum b;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
 	BC_CHECK_NUM(ctxt, a);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -1014,21 +1205,16 @@ BclNumber bcl_irand(BclNumber a) {
 	bc_num_clear(&b);
 	bc_num_init(&b, BC_NUM_DEF_SIZE);
 
-	BC_SIG_UNLOCK;
-
-	bc_num_irand(aptr, &b, &vm.rng);
+	bc_num_irand(aptr, &b, &vm->rng);
 
 err:
-	BC_SIG_MAYLOCK;
 
 	// Eat the operand.
 	bcl_num_dtor(ctxt, a, aptr);
 
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, b, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
@@ -1038,11 +1224,13 @@ err:
  * @param b       The return parameter.
  * @param places  The number of decimal places to generate.
  */
-static void bcl_frandHelper(BcNum *restrict b, size_t places) {
-
+static void
+bcl_frandHelper(BcNum* restrict b, size_t places)
+{
 	BcNum exp, pow, ten;
 	BcDig exp_digs[BC_NUM_BIGDIG_LOG10];
 	BcDig ten_digs[BC_NUM_BIGDIG_LOG10];
+	BcVm* vm = bcl_getspecific();
 
 	// Set up temporaries.
 	bc_num_setup(&exp, exp_digs, BC_NUM_BIGDIG_LOG10);
@@ -1056,38 +1244,35 @@ static void bcl_frandHelper(BcNum *restrict b, size_t places) {
 	// Clear the temporary that might need to grow.
 	bc_num_clear(&pow);
 
-	BC_SIG_LOCK;
-
 	// Initialize the temporary that might need to grow.
 	bc_num_init(&pow, bc_num_powReq(&ten, &exp, 0));
 
-	BC_SETJMP_LOCKED(err);
-
-	BC_SIG_UNLOCK;
+	BC_SETJMP(vm, err);
 
 	// Generate the number.
 	bc_num_pow(&ten, &exp, &pow, 0);
-	bc_num_irand(&pow, b, &vm.rng);
+	bc_num_irand(&pow, b, &vm->rng);
 
 	// Make the number entirely fraction.
 	bc_num_shiftRight(b, places);
 
 err:
-	BC_SIG_MAYLOCK;
 	bc_num_free(&pow);
-	BC_LONGJMP_CONT;
+	BC_LONGJMP_CONT(vm);
 }
 
-BclNumber bcl_frand(size_t places) {
-
+BclNumber
+bcl_frand(size_t places)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum n;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -1095,18 +1280,13 @@ BclNumber bcl_frand(size_t places) {
 	bc_num_clear(&n);
 	bc_num_init(&n, BC_NUM_DEF_SIZE);
 
-	BC_SIG_UNLOCK;
-
 	bcl_frandHelper(&n, places);
 
 err:
-	BC_SIG_MAYLOCK;
 
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, n, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
@@ -1117,49 +1297,47 @@ err:
  * @param b       The return parameter.
  * @param places  The number of decimal places to generate.
  */
-static void bcl_ifrandHelper(BcNum *restrict a, BcNum *restrict b,
-                             size_t places)
+static void
+bcl_ifrandHelper(BcNum* restrict a, BcNum* restrict b, size_t places)
 {
 	BcNum ir, fr;
+	BcVm* vm = bcl_getspecific();
 
 	// Clear the integer and fractional numbers.
 	bc_num_clear(&ir);
 	bc_num_clear(&fr);
 
-	BC_SIG_LOCK;
-
 	// Initialize the integer and fractional numbers.
 	bc_num_init(&ir, BC_NUM_DEF_SIZE);
 	bc_num_init(&fr, BC_NUM_DEF_SIZE);
 
-	BC_SETJMP_LOCKED(err);
+	BC_SETJMP(vm, err);
 
-	BC_SIG_UNLOCK;
-
-	bc_num_irand(a, &ir, &vm.rng);
+	bc_num_irand(a, &ir, &vm->rng);
 	bcl_frandHelper(&fr, places);
 
 	bc_num_add(&ir, &fr, b, 0);
 
 err:
-	BC_SIG_MAYLOCK;
 	bc_num_free(&fr);
 	bc_num_free(&ir);
-	BC_LONGJMP_CONT;
+	BC_LONGJMP_CONT(vm);
 }
 
-BclNumber bcl_ifrand(BclNumber a, size_t places) {
-
+BclNumber
+bcl_ifrand(BclNumber a, size_t places)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *aptr;
+	BcNum* aptr;
 	BcNum b;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 	BC_CHECK_NUM(ctxt, a);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	bc_vec_grow(&ctxt->nums, 1);
 
@@ -1173,34 +1351,31 @@ BclNumber bcl_ifrand(BclNumber a, size_t places) {
 	bc_num_clear(&b);
 	bc_num_init(&b, BC_NUM_DEF_SIZE);
 
-	BC_SIG_UNLOCK;
-
 	bcl_ifrandHelper(aptr, &b, places);
 
 err:
-	BC_SIG_MAYLOCK;
 
 	// Eat the oprand.
 	bcl_num_dtor(ctxt, a, aptr);
 
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, b, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-BclError bcl_rand_seedWithNum(BclNumber n) {
-
+BclError
+bcl_rand_seedWithNum(BclNumber n)
+{
 	BclError e = BCL_ERROR_NONE;
-	BcNum *nptr;
+	BcNum* nptr;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT_ERR(ctxt);
+	BC_CHECK_CTXT_ERR(vm, ctxt);
 	BC_CHECK_NUM_ERR(ctxt, n);
 
-	BC_FUNC_HEADER(err);
+	BC_FUNC_HEADER(vm, err);
 
 	assert(n.i < ctxt->nums.len);
 
@@ -1208,80 +1383,87 @@ BclError bcl_rand_seedWithNum(BclNumber n) {
 
 	assert(nptr != NULL && nptr->num != NULL);
 
-	bc_num_rng(nptr, &vm.rng);
+	bc_num_rng(nptr, &vm->rng);
 
 err:
-	BC_SIG_MAYLOCK;
-	BC_FUNC_FOOTER(e);
-
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
+	BC_FUNC_FOOTER(vm, e);
 	return e;
 }
 
-BclError bcl_rand_seed(unsigned char seed[BCL_SEED_SIZE]) {
-
+BclError
+bcl_rand_seed(unsigned char seed[BCL_SEED_SIZE])
+{
 	BclError e = BCL_ERROR_NONE;
 	size_t i;
 	ulong vals[BCL_SEED_ULONGS];
+	BcVm* vm = bcl_getspecific();
 
-	BC_FUNC_HEADER(err);
+	BC_FUNC_HEADER(vm, err);
 
 	// Fill the array.
-	for (i = 0; i < BCL_SEED_SIZE; ++i) {
-		ulong val = ((ulong) seed[i]) << (((ulong) CHAR_BIT) *
-		                                  (i % sizeof(ulong)));
+	for (i = 0; i < BCL_SEED_SIZE; ++i)
+	{
+		ulong val = ((ulong) seed[i])
+		            << (((ulong) CHAR_BIT) * (i % sizeof(ulong)));
 		vals[i / sizeof(long)] |= val;
 	}
 
-	bc_rand_seed(&vm.rng, vals[0], vals[1], vals[2], vals[3]);
+	bc_rand_seed(&vm->rng, vals[0], vals[1], vals[2], vals[3]);
 
 err:
-	BC_SIG_MAYLOCK;
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	return e;
 }
 
-void bcl_rand_reseed(void) {
-	bc_rand_srand(bc_vec_top(&vm.rng.v));
+void
+bcl_rand_reseed(void)
+{
+	BcVm* vm = bcl_getspecific();
+
+	bc_rand_srand(bc_vec_top(&vm->rng.v));
 }
 
-BclNumber bcl_rand_seed2num(void) {
-
+BclNumber
+bcl_rand_seed2num(void)
+{
 	BclError e = BCL_ERROR_NONE;
 	BcNum n;
 	BclNumber idx;
 	BclContext ctxt;
+	BcVm* vm = bcl_getspecific();
 
-	BC_CHECK_CTXT(ctxt);
+	BC_CHECK_CTXT(vm, ctxt);
 
-	BC_FUNC_HEADER_LOCK(err);
+	BC_FUNC_HEADER(vm, err);
 
 	// Clear and initialize the number.
 	bc_num_clear(&n);
 	bc_num_init(&n, BC_NUM_DEF_SIZE);
 
-	BC_SIG_UNLOCK;
-
-	bc_num_createFromRNG(&n, &vm.rng);
+	bc_num_createFromRNG(&n, &vm->rng);
 
 err:
-	BC_SIG_MAYLOCK;
-	BC_FUNC_FOOTER(e);
+	BC_FUNC_FOOTER(vm, e);
 	BC_MAYBE_SETUP(ctxt, e, n, idx);
 
-	assert(!vm.running && !vm.sig && !vm.sig_lock);
-
 	return idx;
 }
 
-BclRandInt bcl_rand_int(void) {
-	return (BclRandInt) bc_rand_int(&vm.rng);
+BclRandInt
+bcl_rand_int(void)
+{
+	BcVm* vm = bcl_getspecific();
+
+	return (BclRandInt) bc_rand_int(&vm->rng);
 }
 
-BclRandInt bcl_rand_bounded(BclRandInt bound) {
+BclRandInt
+bcl_rand_bounded(BclRandInt bound)
+{
+	BcVm* vm = bcl_getspecific();
+
 	if (bound <= 1) return 0;
-	return (BclRandInt) bc_rand_bounded(&vm.rng, (BcRand) bound);
+	return (BclRandInt) bc_rand_bounded(&vm->rng, (BcRand) bound);
 }
 
 #endif // BC_ENABLE_LIBRARY
